<?php
/**
 * Unit test class for the FunctionCallSignature sniff.
 *
 * PHP version 5
 *
 * @category  PHP
 * @package   PHP_CodeSniffer
 * @author    Greg Sherwood <gsherwood@squiz.net>
 * @author    Marc McIntyre <mmcintyre@squiz.net>
 * @copyright 2006-2014 Squiz Pty Ltd (ABN 77 084 670 600)
 * @license   https://github.com/squizlabs/PHP_CodeSniffer/blob/master/licence.txt BSD Licence
 * @link      http://pear.php.net/package/PHP_CodeSniffer
 */

/**
 * Unit test class for the FunctionCallSignature sniff.
 *
 * A sniff unit test checks a .inc file for expected violations of a single
 * coding standard. Expected errors and warnings are stored in this class.
 *
 * @category  PHP
 * @package   PHP_CodeSniffer
 * @author    Greg Sherwood <gsherwood@squiz.net>
 * @author    Marc McIntyre <mmcintyre@squiz.net>
 * @copyright 2006-2014 Squiz Pty Ltd (ABN 77 084 670 600)
 * @license   https://github.com/squizlabs/PHP_CodeSniffer/blob/master/licence.txt BSD Licence
 * @version   Release: 2.5.1
 * @link      http://pear.php.net/package/PHP_CodeSniffer
 */
class PEAR_Tests_Functions_FunctionCallSignatureUnitTest extends AbstractSniffUnitTest
{


    /**
     * Returns the lines where errors should occur.
     *
     * The key of the array should represent the line number and the value
     * should represent the number of errors that should occur on that line.
     *
     * @param string $testFile The name of the file being tested.
     *
     * @return array<int, int>
     */
    public function getErrorList($testFile='FunctionCallSignatureUnitTest.inc')
    {
        if ($testFile === 'FunctionCallSignatureUnitTest.js') {
            return array(
                    5  => 1,
                    6  => 2,
                    7  => 1,
                    8  => 1,
                    9  => 2,
                    10 => 3,
                    17 => 1,
                    18 => 1,
                    21 => 1,
                    24 => 1,
                    28 => 2,
                    30 => 2,
                    35 => 1,
                    49 => 1,
                    51 => 1,
                    54 => 1,
                    70 => 1,
                    71 => 1,
                   );
        }

        return array(
                5   => 1,
                6   => 2,
                7   => 1,
                8   => 1,
                9   => 2,
                10  => 3,
                17  => 1,
                18  => 1,
                31  => 1,
                34  => 1,
                43  => 2,
                57  => 1,
                59  => 1,
                63  => 1,
                64  => 1,
                82  => 1,
                93  => 1,
                100 => 1,
                106 => 2,
                119 => 1,
                120 => 1,
                129 => 1,
                137 => 1,
                142 => 2,
                171 => 1,
                180 => 1,
                181 => 1,
                194 => 1,
                213 => 2,
                215 => 2,
                274 => 1,
                275 => 1,
               );

    }//end getErrorList()


    /**
     * Returns the lines where warnings should occur.
     *
     * The key of the array should represent the line number and the value
     * should represent the number of warnings that should occur on that line.
     *
     * @return array<int, int>
     */
    public function getWarningList()
    {
        return array();

    }//end getWarningList()


}//end class
