<?php

class ExampleClass
{
	function exampleFunction()	{}

}

	$o = <<<EOF
	this is some text
	this is some text
EOF;

    $correctVar = true;
	$correctVar = false;

class MyClass
{
    /**
	 * Short	description.
	 *
	 * @return	void
	 */
    public function 	myFunction() {	}
}

$str = 'hello
		there';

$foo = array(
	'Česká republika'   => 'Czech republic',
	'România'			=> 'Romania',
	'Magyarország'	   	=> 'Hungary',
);

$var = "$hello	$there";
